/*
 * $Id: stdlib.h 565570 2013-02-08 23:23:49Z ib-builder $
 *
 * stdlib.h -- shim header for g++ support
 *
 * Stu Grossman, March 2007
 *
 * Copyright (c) 2007-2009, Juniper Networks, Inc.
 * All rights reserved.
 */

/*
 * More random stub functions to make libstdc++ compile.  Some stuff, like
 * malloc, free and realloc are actual microkernel functions, and should really
 * be in pfe/include/stdlib.h, but we're not ready to go there yet, since it
 * would involve changing a lot of files (we'd have to get rid of malloc.h).
 */

#ifndef __LIBSTDCXX_EMBEDDED_STDLIB_H__
#define __LIBSTDCXX_EMBEDDED_STDLIB_H__

#ifdef __cplusplus
extern "C" {
#endif

#include "../../../../include/stdlib.h"

typedef struct 
{
  int quot; /* quotient */
  int rem; /* remainder */
} div_t;

typedef struct 
{
  long quot; /* quotient */
  long rem; /* remainder */
} ldiv_t;

typedef struct
{
  long long int quot; /* quotient */
  long long int rem; /* remainder */
} lldiv_t;

#undef abs

void abort(void) __attribute__((__noreturn__));
int abs(int);
int atexit(void (*__func)(void));
double atof(const char *__nptr);
long atol(const char *__nptr);
void *calloc(size_t __nmemb, size_t __size);
div_t div(int __numer, int __denom);
void exit(int __status) __attribute__ ((__noreturn__));
void free(void *);
char *getenv(const char *__string);
long labs(long);
ldiv_t ldiv(long __numer, long __denom);
void *malloc(size_t __size);
int rand(void);
void *realloc(void *__r, size_t __size);
void srand(unsigned __seed);
double strtod(const char *__n, char **__end_PTR);
float strtof(const char *__n, char **__end_PTR);
long strtol(const char *__n, char **__end_PTR, int __base);
int system(const char *__string);

#ifdef __cplusplus
}
#endif

#endif /* __LIBSTDCXX_EMBEDDED_STDLIB_H__ */
