/* Target definitions for GCC for PowerPC for Juniper
   Copyright (C) 2005
   Free Software Foundation, Inc.

This file is part of GCC.

GCC is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GCC is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GCC; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#define POWERPC_JUNOS

#ifdef HAVE_LD_NO_DOT_SYMS
/* New ABI uses a local sym for the function entry point.  */
#undef DOT_SYMBOLS
#define DOT_SYMBOLS 0
#endif

#ifdef HAVE_LD_NO_DOT_SYMS
/* New ABI uses a local sym for the function entry point.  */
extern int dot_symbols;
#undef DOT_SYMBOLS
#define DOT_SYMBOLS dot_symbols
#endif

#undef TARGET_VERSION
#define TARGET_VERSION fprintf (stderr, " (PowerPC Juniper Embedded)");

/* Use a more modern profiler.  Don't allocate the counter word.  Don't emit
   the profiling code before the prologue.  Don't save/restore the static chain
   pointers.  Emit the profiling code as RTL instead of assembly.  */

#undef NO_PROFILE_COUNTERS
#define NO_PROFILE_COUNTERS 1

#undef PROFILE_BEFORE_PROLOGUE

#undef FUNCTION_PROFILER
#define FUNCTION_PROFILER(FILE, LABELNO)

#undef ASM_OUTPUT_REG_PUSH
#undef ASM_OUTPUT_REG_POP

/* See output_profile_hook in rs6000.c.  Call _mcount_internal, passing in
   two arguments: FROMPC and SELFPC.  */
#define PROFILE_HOOK(LABEL) \
do {									\
  if (DEFAULT_ABI == ABI_V4)						\
    {									\
      const char *mcount_name = RS6000_MCOUNT;				\
      int caller_addr_regno = LINK_REGISTER_REGNUM;			\
      rtx arg1;								\
      rtx lr_reg = gen_rtx_REG (Pmode, caller_addr_regno);		\
									\
      arg1 = gen_reg_rtx (Pmode);					\
      emit_move_insn (arg1, lr_reg);					\
      emit_library_call (gen_rtx_SYMBOL_REF (Pmode, mcount_name),	\
			 0, VOIDmode, 1,				\
			 arg1, Pmode);					\
    }									\
  else									\
    output_profile_hook (LABEL);					\
} while (0)

/* This is how to declare the size of a function.  */
#undef	ASM_DECLARE_FUNCTION_SIZE
#define	ASM_DECLARE_FUNCTION_SIZE(FILE, FNAME, DECL)			\
  do									\
    {									\
      if (!flag_inhibit_size_directive)					\
	{								\
	  fputs ("\t.size\t", (FILE));					\
	  if (TARGET_64BIT && DOT_SYMBOLS)				\
	    putc ('.', (FILE));						\
	  assemble_name ((FILE), (FNAME));				\
	  fputs (",.-", (FILE));					\
	  rs6000_output_function_entry (FILE, FNAME);			\
	  putc ('\n', (FILE));						\
	}								\
    }									\
  while (0)

#undef  LINK_SHLIB_SPEC
#define LINK_SHLIB_SPEC "%{shared:-shared} %{!shared: %{static:-static}}"

#undef CPP_OS_FREEBSD_SPEC
#define CPP_OS_FREEBSD_SPEC     "\
          -D__PPC__ -D__ppc__ -D__PowerPC__ -D__powerpc__"

#undef  SIZE_TYPE
#define SIZE_TYPE  "unsigned int"

