/*
 * $Id: libstdc++-ukern.c 705412 2015-03-31 11:59:28Z ib-builder $
 *
 * libstdc++_ukern.c -- g++ support for the microkernel
 *
 * Stu Grossman, January 2007
 *
 * Copyright (c) 2007-2009, Juniper Networks, Inc.
 * All rights reserved.
 */

#include "defs.h"

/*
 * libstdcxx_init
 *
 * Runs C++ static constructors
 *
 * This needs to be done after the heap has been initialized, and should
 * probably be done after console I/O and syslog are setup.  Currently we do it
 * as a module init, which is late enough to satisfy the above requirements.
 * But it might make sense to do it earlier, like in main(), after calling
 * hardware_basic_init().
 */

void libstdcxx_init (void);

extern void (*__CTOR_END__[1])(void);

void
libstdcxx_init (void)
{
    void (**func)(void);

    for (func = __CTOR_END__ - 1; (u_int32_t)*func != -1 ; func--)
	(*func)();
}

/*
 * Stubs for libstdc++
 */
#include "stdio.h"
#include "string.h"
#include "malloc.h"
#include "kdebug.h"
#include "stdlib.h"
#include "math.h"
#include "locale.h"
#include "unistd.h"
#include "sys/stat.h"

void
abort (void)
{
    panic("abort 0x%lx", (unsigned long)cpu_callers_pc());

    /*
     * The following loop is needed to keep gcc happy because panic does not
     * have the noreturn attribute.
     */
    while (1);
}

size_t
fread (void *buf, size_t _size, size_t _n, FILE *file)
{
    panic(__FUNCTION__);
    return 0;
}

size_t
fwrite (const void *buf, size_t size, size_t nmemb, FILE *file)
{
    int i;
    const unsigned char *bufp = buf;

    for (i = 0; i < size * nmemb; i++)
	putchar(bufp[i]);

    return nmemb;
}

int
putc(int c, FILE *file)
{
    return putchar(c);
}

int
fputs (const char *str, FILE *file)
{
    printf("%s", str);

    return 0;
}

int
getc (FILE *file)
{
    panic(__FUNCTION__);
    return 0;
}

int
ungetc (int c, FILE *file)
{
    panic(__FUNCTION__);
    return 0;
}

int
strcoll (const char *s1, const char *s2)
{
    panic(__FUNCTION__);
    return 0;
}

unsigned long
strxfrm (char *dst, const char *src, size_t n)
{
    panic(__FUNCTION__);
    return 0;
}


int
fflush (FILE *stream)
{
    return 0;
}

int
fileno (FILE *file)
{
    panic(__FUNCTION__);
    return 0;
}

FILE *
fdopen (int fd, const char *filename)
{
    panic(__FUNCTION__);
    return 0;
}

FILE *
fopen (const char *_name, const char *_type)
{
    panic(__FUNCTION__);
    return 0;
}

int
fclose (FILE *file)
{
    panic(__FUNCTION__);
    return 0;
}

int
setvbuf (FILE *file, char *buf, int mode, size_t size)
{
    panic(__FUNCTION__);
    return 0;
}

long
ftell (FILE *file)
{
    panic(__FUNCTION__);
    return 0;
}

int
fseek (FILE * file, long offset, int whence)
{
    panic(__FUNCTION__);
    return 0;
}

#undef memmove

void *
memmove (void *dst, const void *src, size_t len)
{
    return memcpy(dst, src, len);
}

double
strtod (const char *__n, char **__end_PTR)
{
    panic(__FUNCTION__);
    return 0;
}

float
strtof (const char *__n, char **__end_PTR)
{
    panic(__FUNCTION__);
    return 0;
}

int
fstat(int fd, struct stat *stat)
{
    panic(__FUNCTION__);
    return 0;
}

char *
setlocale(int category, const char *locale)
{
    panic(__FUNCTION__);
    return 0;
}

int
write(int __fd, const void *__buf, size_t __nbyte)
{
    panic(__FUNCTION__);
    return 0;
}

int
read(int __fd, const void *__buf, size_t __nbyte)
{
    panic(__FUNCTION__);
    return 0;
}

long
lseek(int __fildes, off_t __offset, int __whence)
{
    panic(__FUNCTION__);
    return 0;
}

const char _ctype_[256] = {0};	/* XXX */
